(use gauche.test)
(use file.util)

(test-start "lazy-ffi")

(if (member "." *load-path*) ;; trick to allow in-place test
  (load "lazy-ffi"))
(import lazy-ffi)
(test-module 'lazy-ffi)

(define (serch-dynamic-library regexp)
  (car (directory-list "/usr/lib" :children? #f :add-path? #f
                       :filter (lambda (e) ((string->regexp regexp) e)))))

(define (open-test1)
  ;; XXX OS dependent
  (let* ((dyn-libc (serch-dynamic-library "libc\.[^a-z]*so"))
         (libc (lazy-ffi:open dyn-libc)))
    (if libc #t #f)))

(define (open-test2)
  ;; XXX OS dependent
  (let ((libc (lazy-ffi:open "unknown lib")))
    (if libc #t #f)))

(define (close-test1)
  ;; XXX OS dependent
  (let* ((dyn-libc (serch-dynamic-library "libc\.[^a-z]*so"))
         (libc (lazy-ffi:open dyn-libc)))
    (if (lazy-ffi:close libc) #t #f)))

(define (close-test2)
  ;; XXX OS dependent
  (let* ((dyn-libc (serch-dynamic-library "libc\.[^a-z]*so"))
         (libc (lazy-ffi:open dyn-libc)))
    (lazy-ffi:close libc)
    (if (lazy-ffi:close libc) #t #f)))

(define (function-test1)
  ;; XXX OS dependent
  (let* ((dyn-libc (serch-dynamic-library "libc\.[^a-z]*so"))
         (libc (lazy-ffi:open dyn-libc)))
    (let ((str (lazy-ffi:function libc "strcat" lazy:string "abc" "def")))
      (equal? str "abcdef"))))

(define (function-test2)
  ;; XXX OS dependent
  (let* ((dyn-libc (serch-dynamic-library "libc\.[^a-z]*so"))
         (libc (lazy-ffi:open dyn-libc)))
    (let ((sine (lazy-ffi:function libc "tzset" lazy:void)))
      #t)))

(define (function-test3)
  ;; XXX OS dependent
  (let* ((dyn-libm (serch-dynamic-library "libm\.[^a-z]*so"))
         (libm (lazy-ffi:open dyn-libm)))
    (let ((sine-ret (lazy-ffi:function libm "sin" lazy:double 1.0)))
      (<= (abs (- sine-ret (sin 1.0))) 0.00001))))

(test-section "lazy-ffi:open")
(test* "open success"  #t (open-test1))
(test* "open failure"  #f (open-test2))

(test-section "lazy-ffi:close")
(test* "close success" #t (close-test1))
(test* "close failure" #f (close-test2))

(test-section "lazy-ffi:function")
(test* "libc:strcat"   #t (function-test1))
(test* "libc:tzset"    #t (function-test2))
(test* "libm:sin"      #t (function-test3))

(define libm (lazy-ffi:open "libm.so.2.0"))
(print (lazy-ffi:function libm "sin" lazy:double 1.0))

(test-end)
